@@ -18,7 +18,9 @@ char	primary[11];
 char	*host = NULL;
 char 	*dbase = "test";
 char 	*table = "test";
+char  *subarg = NULL;
 
+void do_substitute(char *subarg, dbhead *dbh);
 inline void strtoupper(char *string);
 inline void strtolower(char *string);
 void do_create(int, char*, dbhead*);
@@ -83,9 +85,63 @@ int check_table(int sock, char *table) {
 void usage(void){
 		printf("dbf2msql %s\n", VERSION);
 		printf("usage:\tdbf2msql\t[-u | -l] [-h hostname] [-p primary key]\n");
+		printf("\t\t\t[-s oldname=newname[,oldname=newname]]\n");
 		printf("\t\t\t-d dbase -t table [-c] [-f] [-v[v]] dbf-file\n");
 }
 
+/* patch submitted by Jeffrey Y. Sue <jysue@aloha.net> */
+/* Provides functionallity for substituting dBase-fieldnames for others */
+/* Mainly for avoiding conflicts between fieldnames and mSQL-reserved */
+/* keywords */
+
+void do_substitute(char *subarg, dbhead *dbh)
+{
+      /* NOTE: subarg is modified in this function */
+      int i,bad;
+      char *p,*oldname,*newname;
+      if (!subarg) {
+              return;
+      }
+      if (verbose>1) {
+              printf("Substituting new field names\n");
+      }
+      /* use strstr instead of strtok because of possible empty tokens */
+      oldname = subarg;
+      while (oldname && strlen(oldname) && (p=strstr(oldname,"=")) ) {
+              *p = '\0';      /* mark end of oldname */
+              newname = ++p;  /* point past \0 of oldname */
+              if (strlen(newname)) {  /* if not an empty string */
+                      p = strstr(newname,",");
+                      if (p) {
+                              *p = '\0';      /* mark end of newname */
+                              p++;    /* point past where the comma was */
+                      }
+              }
+              if (strlen(newname)>=DBF_NAMELEN) {
+                      printf("Truncating new field name %s to %d chars\n",
+                              newname,DBF_NAMELEN-1);
+                      newname[DBF_NAMELEN-1] = '\0';
+              }
+              bad = 1;
+              for (i=0;i<dbh->db_nfields;i++) {
+                      if (strcmp(dbh->db_fields[i].db_name,oldname)==0) {
+                              bad = 0;
+                              strcpy(dbh->db_fields[i].db_name,newname);
+                              if (verbose>1) {
+                                      printf("Substitute old:%s new:%s\n",
+                                              oldname,newname);
+                              }
+                              break;
+                      }
+              }
+              if (bad) {
+                      printf("Warning: old field name %s not found\n",
+                              oldname);
+              }
+              oldname = p;
+      }
+} /* do_substitute */
+
 void do_create(int SQLsock, char *table, dbhead *dbh) {
 	char		*query;
 	char 		t[20];
@@ -107,6 +163,10 @@ void do_create(int SQLsock, char *table, dbhead *dbh) {
 	sprintf(query, "CREATE TABLE %s (", table);
 	length = strlen(query);
 	for ( i = 0; i < dbh->db_nfields; i++) {
+              if (!strlen(dbh->db_fields[i].db_name)) {
+                      continue;
+                      /* skip field if length of name == 0 */
+              }
 		if ((strlen(query) != length)) {
                         strcat(query, ",");
                 }
@@ -212,6 +272,10 @@ void do_inserts(int SQLsock, char *table, dbhead *dbh) {
 				vals[0]='\0';
 
 				for (h = 0; h < dbh->db_nfields; h++) {
+                                      /* if length of fieldname==0, skip it */
+                                      if (!strlen(fields[h].db_name)) {
+                                              continue;
+                                      }
 					if (strlen(keys) != 0) {
 						strcat(keys, ",");
 					}
@@ -279,7 +343,7 @@ int main(int argc, char **argv)
 
 	primary[0] = '\0';
 
-	while ((i = getopt(argc, argv, "flucvh:p:d:t:")) != EOF) {
+	while ((i = getopt(argc, argv, "flucvh:p:d:t:s:")) != EOF) {
 		switch (i) {
 			case 'f':
 				fieldlow=1;
@@ -312,6 +376,9 @@ int main(int argc, char **argv)
 				break;
 			case 't':
 				table = (char *)strdup(optarg);
+                break;
+			case 's':
+				subarg = (char *)strdup(optarg);
 				break;
 			case ':':
 				usage();
@@ -382,6 +449,8 @@ int main(int argc, char **argv)
 		msqlClose(SQLsock);
 		exit(1);
 	}
+/* Substitute field names */
+      do_substitute(subarg, dbh);
 
 	if (!create) {
 		if (!check_table(SQLsock, table)) {
